@@ -866,6 +866,8 @@ keyring_copy( KBPOS *kbpos, int mode, KBNODE root )
 	/* append the new data */
 	kbctx=NULL;
 	while( (node = walk_kbnode( root, &kbctx, 0 )) ) {
+		log_info("build_packet(%d)\n",
+			    node->pkt->pkttype );
 	    if( (rc = build_packet( newfp, node->pkt )) ) {
 		log_error("build_packet(%d) failed: %s\n",
 			    node->pkt->pkttype, g10_errstr(rc) );
